package io.sixminutes.ridicule

import android.accessibilityservice.AccessibilityService
import android.content.Intent
import android.graphics.Rect
import android.os.Binder
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.accessibility.AccessibilityEvent
import android.view.accessibility.AccessibilityNodeInfo
import android.widget.Toast
import androidx.core.view.accessibility.AccessibilityNodeInfoCompat
import kotlinx.coroutines.delay
import java.util.LinkedList

class MyAccessibilityService : AccessibilityService() {
    companion object {
        private const val TAG = "AccessibilityService"
        @Volatile private var instance: MyAccessibilityService? = null

        // 使用双重校验锁保证线程安全
        fun getInstance(): MyAccessibilityService? = instance ?: synchronized(this) {
            instance ?: null
        }
    }

    private val binder = MyBinder()
    private var targetAppPackageName: String? = null  // 目标应用的包名

    inner class MyBinder : Binder() {
        fun getService(): MyAccessibilityService = this@MyAccessibilityService
    }

    override fun onServiceConnected() {
        super.onServiceConnected()
        instance = this
        Log.d(TAG, "Service connected")
        startMainActivitySafely()  // 注意：无障碍服务启动Activity需谨慎
    }

    override fun onDestroy() {
        cleanUpResources()
        super.onDestroy()
    }

    override fun onInterrupt() {
        Log.w(TAG, "Service interrupted")
    }

    override fun onAccessibilityEvent(event: AccessibilityEvent?) {
        event?.let {
            Log.v(TAG, "Event received: ${event.eventType}")
            // 添加实际的事件处理逻辑
            processAccessibilityEvent(event)
        }
    }

    override fun onUnbind(intent: Intent?): Boolean {
        instance = null
        return super.onUnbind(intent)
    }

    /**
     * 模拟点击屏幕上的指定坐标
     * @param x 点击的X坐标
     * @param y 点击的Y坐标
     */
    fun simulateTap(x: Int, y: Int) {
        var root: AccessibilityNodeInfo? = null
        val tempRect = Rect() // 复用 Rect 对象提升性能

        try {
            root = rootInActiveWindow ?: run {
                Log.e(TAG, "No root window available")
                return
            }

            val wrappedRoot = AccessibilityNodeInfoCompat.wrap(root)
            try {
                val matchingNodes = findNodesByCondition(
                    root = wrappedRoot,
                    predicate = { node ->
                        // 正确使用 bounds 判断逻辑
                        node.isClickable &&
                                node.isVisibleToUser() &&
                                node.getBoundsInScreen(tempRect).let {
                                    !tempRect.isEmpty && tempRect.contains(x, y)
                                }
                    }
                )

                matchingNodes.firstOrNull()?.let { targetNode ->
                    if (targetNode.performAction(AccessibilityNodeInfo.ACTION_CLICK)) {
                        Log.d(TAG, "Tap performed at ($x, $y)")
                    } else {
                        Log.w(TAG, "Failed to perform click action")
                    }
                } ?: Log.w(TAG, "No clickable node found at ($x, $y)")
            } finally {
                wrappedRoot.recycle()
            }
        } catch (e: Exception) {
            Log.e(TAG, "Simulate tap error", e)
        } finally {
            root?.recycle()
        }
    }

    /**
     * 根据条件查找节点
     * @param root 根节点
     * @param predicate 判断条件
     * @return 符合条件的节点列表
     */
    private fun findNodesByCondition(
        root: AccessibilityNodeInfoCompat,
        predicate: (AccessibilityNodeInfoCompat) -> Boolean
    ): List<AccessibilityNodeInfoCompat> {
        val result = mutableListOf<AccessibilityNodeInfoCompat>()
        val queue = LinkedList<AccessibilityNodeInfoCompat>().apply { add(root) }
        val tempRect = Rect() // 复用 Rect 对象提升性能

        while (queue.isNotEmpty()) {
            val node = queue.poll()
            try {
                // 统一使用带参数的 bounds 获取方式
                node.getBoundsInScreen(tempRect)
                if (predicate(node)) result.add(node)

                // 遍历子节点
                for (i in 0 until node.childCount) {
                    node.getChild(i)?.let {
                        queue.add(AccessibilityNodeInfoCompat.wrap(it.unwrap()))
                    }
                }
            } finally {
                node.recycle()
            }
        }
        return result
    }

    /**
     * 查找并启动指定应用
     * @param appName 应用名称
     */
    /**
     * 查找并启动指定应用
     * @param appName 应用名称
     */
    fun findAndLaunchApp(appName: String) {
        try {
            rootInActiveWindow?.let { rawRoot ->
                // 1. 包装成 Compat 对象
                val wrappedRoot = AccessibilityNodeInfoCompat.wrap(rawRoot)
                try {
                    // 2. 显式传递参数
                    val matchingNodes = findNodesByCondition(
                        root = wrappedRoot,
                        predicate = { node ->
                            // 3. 增强文本匹配逻辑
                            node.text?.toString().orEmpty().contains(appName, true) ||
                                    node.contentDescription?.toString().orEmpty().contains(appName, true)
                        }
                    )

                    // 4. 处理匹配结果
                    matchingNodes.firstOrNull()?.let { targetNode ->
                        if (targetNode.performAction(AccessibilityNodeInfo.ACTION_CLICK)) {
                            Log.d(TAG, "Successfully launched: $appName")

                            // 启动重试机制
                            retryToDetectTargetApp(10, 3000) { success ->
                                if (success) {
                                    Log.d(TAG, "Target app package name detected: $targetAppPackageName")
                                } else {
                                    Log.e(TAG, "Failed to detect target app after 10 retries")
                                    showToast("Failed to launch $appName")
                                }
                            }
                        } else {
                            Log.w(TAG, "Failed to click app: $appName")
                        }
                        targetNode.recycle()
                    } ?: Log.w(TAG, "App not found: $appName")
                } finally {
                    wrappedRoot.recycle() // 确保回收资源
                }
            } ?: Log.e(TAG, "No root window available")
        } catch (e: Exception) {
            Log.e(TAG, "findAndLaunchApp error", e)
        }
    }

    /**
     * 重试检测目标应用
     * @param maxRetries 最大重试次数
     * @param delayMillis 每次重试的延时（毫秒）
     * @param onResult 结果回调，true 表示成功，false 表示失败
     */
    private fun retryToDetectTargetApp(maxRetries: Int, delayMillis: Long, onResult: (Boolean) -> Unit) {
        var retryCount = 0

        val retryRunnable = object : Runnable {
            override fun run() {
                rootInActiveWindow?.let { currentRoot ->
                    val packageName = currentRoot.packageName?.toString()
                    if (packageName != null && packageName != targetAppPackageName) {
                        targetAppPackageName = packageName
                        onResult(true) // 成功检测到目标应用
                        return
                    }
                }

                retryCount++
                if (retryCount < maxRetries) {
                    Handler(Looper.getMainLooper()).postDelayed(this, delayMillis) // 继续重试
                } else {
                    onResult(false) // 重试次数用尽，失败
                }
            }
        }

        // 启动第一次重试
        Handler(Looper.getMainLooper()).postDelayed(retryRunnable, delayMillis)
    }

    /**
     * 显示 Toast 消息
     * @param message 要显示的消息
     */
    private fun showToast(message: String) {
        Handler(Looper.getMainLooper()).post {
            Toast.makeText(this, message, Toast.LENGTH_LONG).show()
        }
    }

    /**
     * 处理无障碍事件
     * @param event 无障碍事件
     */
    private fun processAccessibilityEvent(event: AccessibilityEvent) {
        // 添加具体的事件处理逻辑
        when (event.eventType) {
            AccessibilityEvent.TYPE_WINDOW_STATE_CHANGED -> {
                Log.d(TAG, "Window state changed: ${event.packageName}")
                Log.d(TAG, "target app package name: $targetAppPackageName")
                // 检测当前活动的应用是否发生了变化
                event.packageName?.let { currentPackageName ->
                    if (targetAppPackageName != null && currentPackageName != targetAppPackageName) {
                        Log.d(TAG, "Detected app switch from $targetAppPackageName to $currentPackageName")
                        retryBackToTargetApp(10, 1000) { success ->
                            if (success) {
                                Log.d(TAG, "Successfully returned to $targetAppPackageName")
                            } else {
                                Log.e(TAG, "Failed to return to $targetAppPackageName after 10 retries")
                                showToast("Failed to return to target app")
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 重试返回目标应用
     * @param maxRetries 最大重试次数
     * @param delayMillis 每次重试的延时（毫秒）
     * @param onResult 结果回调，true 表示成功，false 表示失败
     */
    private fun retryBackToTargetApp(maxRetries: Int, delayMillis: Long, onResult: (Boolean) -> Unit) {
        var retryCount = 0

        val retryRunnable = object : Runnable {
            override fun run() {
                // 检测当前窗口是否已经返回到目标应用
                rootInActiveWindow?.let { currentRoot ->
                    val packageName = currentRoot.packageName?.toString()
                    if (packageName == targetAppPackageName) {
                        onResult(true) // 成功返回到目标应用
                        return
                    }
                }

                // 尝试点击返回按钮
                performGlobalAction(GLOBAL_ACTION_BACK)
                Log.d(TAG, "Performed back action, retry count: $retryCount")

                retryCount++
                if (retryCount < maxRetries) {
                    Handler(Looper.getMainLooper()).postDelayed(this, delayMillis) // 继续重试
                } else {
                    onResult(false) // 重试次数用尽，失败
                }
            }
        }

        // 启动第一次重试
        Handler(Looper.getMainLooper()).postDelayed(retryRunnable, delayMillis)
    }


    /**
     * 安全启动主Activity
     */
    private fun startMainActivitySafely() {
        try {
            startActivity(
                Intent(this, MainActivity::class.java)
                    .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
            )
        } catch (e: Exception) {
            Log.e(TAG, "Failed to start MainActivity", e)
        }
    }

    /**
     * 清理资源
     */
    private fun cleanUpResources() {
        instance = null
        targetAppPackageName = null
    }

    /**
     * 遍历节点树
     * @param block 处理每个节点的函数
     */
    private fun AccessibilityNodeInfoCompat.traverse(block: (AccessibilityNodeInfoCompat) -> Boolean) {
        if (!block(this)) return
        for (i in 0 until childCount) {
            getChild(i)?.let { child ->
                child.traverse(block)
                child.recycle()  // 确保每个子节点都被回收
            }
        }
    }
}